# -*- encoding=utf-8 -*-
# Original sources Copyright (c) 2006 by Tim Sutton
#
# ported to Python by Martin Dobias
#
# licensed under the terms of GNU GPL 2
#
# Adaptado por German Carrillo
# GeoTux
# http://geotux.tuxfamily.org
#

import sys
from os.path import isfile

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from qgis.core import *
from qgis.gui import *

from visor_shapefiles_ui import Ui_MainWindow
from cargue_postgis import DialogoCarguePostGIS

qgis_prefix = "/usr"


class VisorShapefiles(QMainWindow, Ui_MainWindow):
    """ Clase Visor de Shapefiles. """
    def __init__(self):
        """ Inicializador de la clase Visor de Shapefiles. """
        QMainWindow.__init__(self)

        # Requerido por Qt4 para inicializar la UI
        self.setupUi(self)

		# Fijar el titulo
        self.setWindowTitle("GeoTux, Visor de Shapefiles con QGis")

        # Crear el Map Canvas
        self.canvas = QgsMapCanvas()
        self.canvas.setCanvasColor(QColor(255,255,255))
        self.canvas.enableAntiAliasing(True)
        self.canvas.useImageToRender(False)
        self.canvas.show()

        # Se agrega el Map canvas a la ventana principal, dentro del marco
        self.layout = QVBoxLayout(self.frame)
        self.layout.addWidget(self.canvas)

        # Crear los comportamientos de los botones
        self.actionAddLayer = QAction(QIcon(':/botones/imagenes/mActionAddOgrLayer.png'),
            "Agregar capa", self.frame)
        self.connect(self.actionAddLayer, SIGNAL("activated()"), self.addLayer)

        self.actionZoomIn = QAction(QIcon(':/botones/imagenes/mActionZoomIn.png'),
            "Acercar", self.frame)
        self.connect(self.actionZoomIn, SIGNAL("activated()"), self.zoomIn)

        self.actionZoomOut = QAction(QIcon(':/botones/imagenes/mActionZoomOut.png'),
            "Alejar", self.frame)
        self.connect(self.actionZoomOut, SIGNAL("activated()"), self.zoomOut)

        self.actionPan = QAction(QIcon(':/botones/imagenes/mActionPan.png'), "Panear", self.frame)
        self.connect(self.actionPan, SIGNAL("activated()"), self.pan)

        self.actionZoomFull = QAction(QIcon(':/botones/imagenes/mActionZoomFullExtent.png'),
            "Vista completa", self.frame)
        self.connect(self.actionZoomFull, SIGNAL("activated()"), self.zoomFull)

        self.actionCargarPostGIS = QAction(QIcon(':/botones/imagenes/mActionAddLayer.png'),
            "Agregar capa de PostGIS", self.frame)
        self.connect(self.actionCargarPostGIS, SIGNAL("activated()"),
            self.cargarPostGIS)

        # Crear una toolbar
        self.toolbar = self.addToolBar("Map")
        self.toolbar.addAction(self.actionAddLayer)
        self.toolbar.addAction(self.actionCargarPostGIS)
        self.toolbar.addAction(self.actionZoomIn)
        self.toolbar.addAction(self.actionZoomOut)
        self.toolbar.addAction(self.actionPan)
        self.toolbar.addAction(self.actionZoomFull)

        # Crear las herramientas (tools) para el mapa
        self.toolPan = QgsMapToolPan(self.canvas)
        self.toolZoomIn = QgsMapToolZoom(self.canvas, False) # false = Acercar
        self.toolZoomOut = QgsMapToolZoom(self.canvas, True) # true = Alejar

        self.layers = [] # Lista de capas del canvas, para poder agregar capas a las existentes

    def zoomIn(self):
        """ Acercar """
        self.canvas.setMapTool(self.toolZoomIn)

    def zoomOut(self):
        """ Alejar """
        self.canvas.setMapTool(self.toolZoomOut)

    def pan(self):
        """ Panear """
        self.canvas.setMapTool(self.toolPan)

    def zoomFull(self):
        """ Vista total """
        self.canvas.zoomToFullExtent()

    def addLayer(self):
        """ Agregar capa al canvas """
        layerPath = QFileDialog.getOpenFileName(self, "Abrir shapefile", ".", "Shapefiles (*.shp)")
        layerInfo = QFileInfo(layerPath)
        layerProvider = "ogr"

        if isfile( unicode( layerPath ) ):
            # Crear el layer
            layer = QgsVectorLayer(layerPath, layerInfo.fileName(), layerProvider)

            self.cargarCapa( layer )

    def cargarPostGIS(self):
        """ Agregar capa de PostGIS al canvas """
        dialogoPg = DialogoCarguePostGIS.obtenerInstancia(self) # (Parent)
        dialogoPg.mostrar()

        if dialogoPg.resultado:
            uri = QgsDataSourceURI()
            # Host, puerto, bd, usuario, password
            uri.setConnection(dialogoPg.servidor, dialogoPg.puerto, dialogoPg.bd,
                dialogoPg.usuario, dialogoPg.contrasena)

            # Esquema, tabla, columna geometria y WHERE opcional
            uri.setDataSource(dialogoPg.esquema, dialogoPg.tabla, dialogoPg.geom)

            layer = QgsVectorLayer(uri.uri(), str(dialogoPg.tabla).capitalize(),
                "postgres")
    
            self.cargarCapa( layer )

    def cargarCapa( self, capa ):
        """ Carga una capa al canvas """
        if capa.isValid():
            # Agregar el layer al registro
            QgsMapLayerRegistry.instance().addMapLayer(capa);

            # Fijar el extent al extent del primer layer cargado
            if self.canvas.layerCount() == 0:
                self.canvas.setExtent(capa.extent())

            # Fijar el conjunto de capas (LayerSet) para el map canvas
            self.layers.insert(0, QgsMapCanvasLayer(capa))
            self.canvas.setLayerSet(self.layers)
        else:
            if capa.dataProvider() == "postgres":
                msg = QMessageBox(QMessageBox.Warning, 'Advertencia', 'No se ' +
                    'ha podido establecer la conexin con \nla base de datos ' +
                    'que especificaste.\n\nRevisa los parmetros de conexin.',
                    QMessageBox.NoButton, self)
                msg.addButton('Aceptar', QMessageBox.AcceptRole)
                msg.exec_()        


def main(argv):

    app = QApplication(argv)

	# Inicializar las librerias de QGIS
    QgsApplication.setPrefixPath(qgis_prefix, True)
    QgsApplication.initQgis()

    # Crear y mostrar la ventana principal
    wnd = VisorShapefiles()
    wnd.move(100,100)
    wnd.show()

    # Ejecutar un loop para correr el programa
    retval = app.exec_()

	# Salir
    QgsApplication.exitQgis()
    sys.exit(retval)


if __name__ == "__main__":
	# Si el modulo es importado __name__= VisorShapefiles  ,
	#    si es ejecutado: __name__ = __main__
    main(sys.argv) # Crear la aplicacion Qt
