# -*- coding: utf-8 -*-
import re

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from conexion_postgis import ConexionPgSQL
from recursos import *

class DialogoCarguePostGIS(QDialog):
    """ Provee la interfaz de la herramienta Cargar capa PostGIS """

    instancia = None # Variable para controlar la nica instancia

    class SingletonHelper:
        """ Provee un mtodo factory para el Singleton DialogoCarguePostGIS """
        def __call__( self, *args, **kw ) :

            if DialogoCarguePostGIS.instancia is None :
                object = DialogoCarguePostGIS(args[0]) # Parent
                DialogoCarguePostGIS.instancia = object

            return DialogoCarguePostGIS.instancia

    obtenerInstancia = SingletonHelper() # Mtodo para obtener la instancia

    def __init__(self, parent=None):

        if not DialogoCarguePostGIS.instancia == None :
            raise RuntimeError, 'Solo se permite una instancia de esta clase!'

        QDialog.__init__(self, parent) # Inicializar la clase QDialog

        # Diagramacin de la Interfaz
        self.setWindowTitle('Cargar capa de PostGIS')

        lbl_servidor = QLabel('Servidor:')
        lbl_puerto = QLabel('Puerto:')
        lbl_bd = QLabel('Base de datos:')
        lbl_usuario = QLabel('Usuario:')
        lbl_contrasena = QLabel('Contrasea:')
        lbl_tabla = QLabel('Tabla:')

        self.__servidorEdit = QLineEdit("localhost")
        self.__puertoEdit = QLineEdit("5432")
        self.__bdEdit = QLineEdit("trasmisig")
        self.__usuarioEdit = QLineEdit("usuario")
        self.__contrasenaEdit = QLineEdit("usuario")
        self.__contrasenaEdit.setEchoMode(QLineEdit.Password) # Ocultar password

        self.cboTabla = QComboBox() # ComboBox para mostrar tablas disponibles
        self.cboTabla.setObjectName("cboTabla")

        self.btnConectar = QPushButton(QIcon(":/imgs/imagenes/conectar.png"),
            "Conectar")

        buttonBox = QDialogButtonBox()
        self.btnAceptar = buttonBox.addButton('Aceptar',
            QDialogButtonBox.AcceptRole)
        self.btnAceptar.setEnabled(False)
        buttonBox.addButton('Cancelar', QDialogButtonBox.RejectRole)

        QObject.connect(self.btnConectar, SIGNAL("clicked()"), self.__conectar)
        QObject.connect(buttonBox, SIGNAL("accepted()"), self.__aceptar)
        QObject.connect(buttonBox, SIGNAL("rejected()"), self.cerrar)

        grid = QGridLayout()
        grid.setSpacing(0)

        grid.addWidget(lbl_servidor, 1, 0)
        grid.addWidget(self.__servidorEdit, 1, 1)

        grid.addWidget(lbl_puerto, 2, 0)
        grid.addWidget(self.__puertoEdit, 2, 1)

        grid.addWidget(lbl_bd, 3, 0)
        grid.addWidget(self.__bdEdit, 3, 1)

        grid.addWidget(lbl_usuario, 4, 0)
        grid.addWidget(self.__usuarioEdit, 4, 1)

        grid.addWidget(lbl_contrasena, 5, 0)
        grid.addWidget(self.__contrasenaEdit, 5, 1)

        grid.setRowStretch(6,1)

        grid.addWidget(self.btnConectar, 7, 0,1,2, Qt.AlignCenter)

        grid.setRowStretch(8,1)

        grid.addWidget(lbl_tabla, 9, 0)
        grid.addWidget(self.cboTabla, 9, 1)

        grid.setRowStretch(10,1)

        grid.addWidget(buttonBox,11,0,1,2, Qt.AlignCenter)

        self.setLayout(grid)
        self.setFixedSize(260, 275)

        self.resultado = False # Sirve para conocer si se acept o cancel

    def __conectar(self):
        """ Valida los parmetros de conexin y carga las capas disponibles """
        validacion_puerto = re.compile("^(\d{4})$")
        validacion_cadenas = re.compile("^(\S)*\S$") #No espacios, len(cadena)>0

        self.cboTabla.clear() # Inicializar cboTabla

        if (validacion_cadenas.match(str(self.__servidorEdit.text())) and
            validacion_puerto.match(str(self.__puertoEdit.text())) and
            validacion_cadenas.match(str(self.__bdEdit.text())) and
            validacion_cadenas.match(str(self.__usuarioEdit.text())) and
            validacion_cadenas.match(str(self.__contrasenaEdit.text()))):

            self.servidor = self.__servidorEdit.text()
            self.puerto = self.__puertoEdit.text()
            self.bd = self.__bdEdit.text()
            self.usuario = self.__usuarioEdit.text()
            self.contrasena = self.__contrasenaEdit.text()

            # Cargar capas disponibles en la BD
            conn = ConexionPgSQL(self.servidor, self.puerto,
                self.bd, self.usuario, self.contrasena)

            if conn.conectar():
                self.aDatosBD = conn.consultarCapas()

                if self.aDatosBD:
                    for i in range(0, len(self.aDatosBD)):

                        if self.aDatosBD[i][3] == "POINT":
                            icono= QIcon(":/imgs/imagenes/punto.png")

                        elif self.aDatosBD[i][3] == "LINESTRING":
                            icono= QIcon(":/imgs/imagenes/linea.png")

                        elif self.aDatosBD[i][3] == "POLYGON":
                            icono= QIcon(":/imgs/imagenes/poligono.png")

                        self.cboTabla.addItem(icono, self.aDatosBD[i][1])

                    self.btnAceptar.setEnabled(True) # Habilitar
                else:
                    self.btnAceptar.setEnabled(False) # Deshabilitar
                    msg = QMessageBox(QMessageBox.Warning, 'Advertencia',
                        'No existen tablas espaciales en la base de datos que' +
                        '\npuedas visualizar con los parmetros ingresados.',
                        QMessageBox.NoButton, self)
                    msg.addButton('Aceptar', QMessageBox.AcceptRole)
                    msg.exec_()
            else:
                self.btnAceptar.setEnabled(False) # Deshabilitar
                msg = QMessageBox(QMessageBox.Warning, 'Advertencia',
                    'No se ha podido establecer la conexin con \nla base ' +
                    'de datos que especificaste.\n\nRevisa los parmetros ' +
                    'de conexin.', QMessageBox.NoButton, self)
                msg.addButton('Aceptar', QMessageBox.AcceptRole)
                msg.exec_()
        else:
            self.btnAceptar.setEnabled(False) # Deshabilitar
            msg = QMessageBox(QMessageBox.Warning, 'Advertencia1',
                'Has llenado mal algn campo, por favor revisa.',
                QMessageBox.NoButton, self)
            msg.addButton('Aceptar', QMessageBox.AcceptRole)
            msg.exec_()

    def __aceptar(self):
        """ SLOT para el click en el botn Aceptar """
        self.resultado = True # Aceptado

        # Esta asignacin permite validar cualquier cambio posterior a la conexion
        self.servidor = self.__servidorEdit.text()
        self.puerto = self.__puertoEdit.text()
        self.bd = self.__bdEdit.text()
        self.usuario = self.__usuarioEdit.text()
        self.contrasena = self.__contrasenaEdit.text()

        # Datos de la tabla a cargar
        self.esquema = self.aDatosBD[self.cboTabla.currentIndex()][0]
        self.tabla = self.aDatosBD[self.cboTabla.currentIndex()][1]
        self.geom = self.aDatosBD[self.cboTabla.currentIndex()][2]
        self.close()

    def mostrar(self):
        self.exec_()

    def cerrar(self):
        self.resultado = False # Cancelado
        self.close()